@@ -273,17 +273,13 @@ enum
 	PROP_SETTINGS,
 	PROP_HIGHLIGHT,
 	PROP_OCCURRENCES_COUNT,
-	PROP_REGEX_ERROR
+	PROP_REGEX_ERROR,
+	PROP_REGEX_STATE
 };
 
 struct _GtkSourceSearchContextPrivate
 {
-	/* Weak ref to the buffer. The buffer has also a weak ref to the search
-	 * context. A strong ref in either direction would prevent the pointed
-	 * object to be finalized.
-	 */
 	GtkTextBuffer *buffer;
-
 	GtkSourceSearchSettings *settings;
 
 	/* The tag to apply to search occurrences. Even if the highlighting is
@@ -291,12 +287,6 @@ struct _GtkSourceSearchContextPrivate
 	 */
 	GtkTextTag *found_tag;
 
-	/* A reference to the tag table where the found_tag is added. The sole
-	 * purpose is to remove the found_tag in dispose(). We can not rely on
-	 * 'buffer' since it is a weak reference.
-	 */
-	GtkTextTagTable *tag_table;
-
 	/* The region to scan and highlight. If NULL, the scan is finished. */
 	GtkTextRegion *scan_region;
 
@@ -318,6 +308,7 @@ struct _GtkSourceSearchContextPrivate
 
 	GRegex *regex;
 	GError *regex_error;
+	GtkSourceRegexSearchState regex_state;
 
 	gint occurrences_count;
 	gulong idle_scan_id;
@@ -507,7 +498,7 @@ clear_task (GtkSourceSearchContext *search)
 {
 	if (search->priv->task_region != NULL)
 	{
-		gtk_text_region_destroy (search->priv->task_region);
+		gtk_text_region_destroy (search->priv->task_region, TRUE);
 		search->priv->task_region = NULL;
 	}
 
@@ -530,13 +521,13 @@ clear_search (GtkSourceSearchContext *search)
 {
 	if (search->priv->scan_region != NULL)
 	{
-		gtk_text_region_destroy (search->priv->scan_region);
+		gtk_text_region_destroy (search->priv->scan_region, TRUE);
 		search->priv->scan_region = NULL;
 	}
 
 	if (search->priv->high_priority_region != NULL)
 	{
-		gtk_text_region_destroy (search->priv->high_priority_region);
+		gtk_text_region_destroy (search->priv->high_priority_region, TRUE);
 		search->priv->high_priority_region = NULL;
 	}
 
@@ -553,6 +544,12 @@ clear_search (GtkSourceSearchContext *search)
 		g_object_notify (G_OBJECT (search), "regex-error");
 	}
 
+	if (search->priv->regex_state != GTK_SOURCE_REGEX_SEARCH_NO_ERROR)
+	{
+		search->priv->regex_state = GTK_SOURCE_REGEX_SEARCH_NO_ERROR;
+		g_object_notify (G_OBJECT (search), "regex-state");
+	}
+
 	clear_task (search);
 
 	search->priv->occurrences_count = 0;
@@ -728,7 +725,11 @@ basic_forward_regex_search (GtkSourceSearchContext *search,
 
 	if (search->priv->regex_error != NULL)
 	{
+		search->priv->regex_state = GTK_SOURCE_REGEX_SEARCH_MATCHING_ERROR;
+
 		g_object_notify (G_OBJECT (search), "regex-error");
+		g_object_notify (G_OBJECT (search), "regex-state");
+
 		found = FALSE;
 	}
 
@@ -866,7 +867,11 @@ basic_backward_regex_search (GtkSourceSearchContext *search,
 
 	if (search->priv->regex_error != NULL)
 	{
+		search->priv->regex_state = GTK_SOURCE_REGEX_SEARCH_MATCHING_ERROR;
+
 		g_object_notify (G_OBJECT (search), "regex-error");
+		g_object_notify (G_OBJECT (search), "regex-state");
+
 		found = FALSE;
 	}
 
@@ -1001,7 +1006,7 @@ smart_forward_search_async_step (GtkSourceSearchContext *search,
 
 		if (region != NULL)
 		{
-			gtk_text_region_destroy (region);
+			gtk_text_region_destroy (region, TRUE);
 		}
 
 		while (basic_forward_search (search, &iter, &match_start, &match_end, &limit))
@@ -1045,7 +1050,7 @@ smart_forward_search_async_step (GtkSourceSearchContext *search,
 
 	if (search->priv->task_region != NULL)
 	{
-		gtk_text_region_destroy (search->priv->task_region);
+		gtk_text_region_destroy (search->priv->task_region, TRUE);
 	}
 
 	search->priv->task_region = region;
@@ -1137,7 +1142,7 @@ smart_backward_search_async_step (GtkSourceSearchContext *search,
 
 		if (region != NULL)
 		{
-			gtk_text_region_destroy (region);
+			gtk_text_region_destroy (region, TRUE);
 		}
 
 		while (basic_backward_search (search, &iter, &match_start, &match_end, &limit))
@@ -1181,7 +1186,7 @@ smart_backward_search_async_step (GtkSourceSearchContext *search,
 
 	if (search->priv->task_region != NULL)
 	{
-		gtk_text_region_destroy (search->priv->task_region);
+		gtk_text_region_destroy (search->priv->task_region, TRUE);
 	}
 
 	search->priv->task_region = region;
@@ -1285,7 +1290,7 @@ adjust_subregion (GtkSourceSearchContext *search,
 
 			if (region != NULL)
 			{
-				gtk_text_region_destroy (region);
+				gtk_text_region_destroy (region, TRUE);
 			}
 		}
 	}
@@ -1338,7 +1343,7 @@ adjust_subregion (GtkSourceSearchContext *search,
 
 			if (region != NULL)
 			{
-				gtk_text_region_destroy (region);
+				gtk_text_region_destroy (region, TRUE);
 			}
 		}
 	}
@@ -1350,9 +1355,8 @@ adjust_subregion (GtkSourceSearchContext *search,
 	});
 }
 
-/* Do not take into account the scan_region. Take the result with a grain of
- * salt. You should verify before or after calling this function that the
- * region has been scanned, to be sure that the returned occurrence is correct.
+/* Do not take into account the scan_region. Search just with
+ * forward_to_tag_toggle(), and checks that it is not an old match.
  */
 static gboolean
 smart_forward_search_without_scanning (GtkSourceSearchContext *search,
@@ -1460,7 +1464,7 @@ remove_occurrences_in_range (GtkSourceSearchContext *search,
 
 			if (region != NULL)
 			{
-				gtk_text_region_destroy (region);
+				gtk_text_region_destroy (region, TRUE);
 			}
 		}
 
@@ -1657,7 +1661,7 @@ resume_task (GtkSourceSearchContext *search)
 
 	if (search->priv->task_region != NULL)
 	{
-		gtk_text_region_destroy (search->priv->task_region);
+		gtk_text_region_destroy (search->priv->task_region, TRUE);
 		search->priv->task_region = NULL;
 	}
 
@@ -1707,7 +1711,7 @@ idle_scan_normal_search (GtkSourceSearchContext *search)
 		 */
 		scan_all_region (search, search->priv->high_priority_region);
 
-		gtk_text_region_destroy (search->priv->high_priority_region);
+		gtk_text_region_destroy (search->priv->high_priority_region, TRUE);
 		search->priv->high_priority_region = NULL;
 
 		return G_SOURCE_CONTINUE;
@@ -1729,7 +1733,7 @@ idle_scan_normal_search (GtkSourceSearchContext *search)
 
 		if (search->priv->scan_region != NULL)
 		{
-			gtk_text_region_destroy (search->priv->scan_region);
+			gtk_text_region_destroy (search->priv->scan_region, TRUE);
 			search->priv->scan_region = NULL;
 		}
 
@@ -1794,7 +1798,7 @@ regex_search_handle_high_priority_region (GtkSourceSearchContext *search)
 		gtk_text_region_iterator_next (&region_iter);
 	}
 
-	gtk_text_region_destroy (region);
+	gtk_text_region_destroy (region, TRUE);
 }
 
 /* Returns TRUE if the segment is finished, and FALSE on partial match. */
@@ -1913,7 +1917,10 @@ regex_search_scan_segment (GtkSourceSearchContext *search,
 
 	if (search->priv->regex_error != NULL)
 	{
+		search->priv->regex_state = GTK_SOURCE_REGEX_SEARCH_MATCHING_ERROR;
+
 		g_object_notify (G_OBJECT (search), "regex-error");
+		g_object_notify (G_OBJECT (search), "regex-state");
 	}
 
 	if (g_match_info_is_partial_match (match_info))
@@ -2012,7 +2019,7 @@ idle_scan_regex_search (GtkSourceSearchContext *search)
 	{
 		regex_search_handle_high_priority_region (search);
 
-		gtk_text_region_destroy (search->priv->high_priority_region);
+		gtk_text_region_destroy (search->priv->high_priority_region, TRUE);
 		search->priv->high_priority_region = NULL;
 
 		return G_SOURCE_CONTINUE;
@@ -2042,7 +2049,7 @@ idle_scan_regex_search (GtkSourceSearchContext *search)
 
 		if (search->priv->scan_region != NULL)
 		{
-			gtk_text_region_destroy (search->priv->scan_region);
+			gtk_text_region_destroy (search->priv->scan_region, TRUE);
 			search->priv->scan_region = NULL;
 		}
 
@@ -2103,7 +2110,7 @@ smart_forward_search_step (GtkSourceSearchContext *search,
 	{
 		if (region != NULL)
 		{
-			gtk_text_region_destroy (region);
+			gtk_text_region_destroy (region, TRUE);
 		}
 
 		while (basic_forward_search (search, &iter, match_start, match_end, &limit))
@@ -2132,7 +2139,7 @@ smart_forward_search_step (GtkSourceSearchContext *search,
 		scan_region_forward (search, region);
 	}
 
-	gtk_text_region_destroy (region);
+	gtk_text_region_destroy (region, TRUE);
 
 	return FALSE;
 }
@@ -2202,7 +2209,7 @@ smart_backward_search_step (GtkSourceSearchContext *search,
 	{
 		if (region != NULL)
 		{
-			gtk_text_region_destroy (region);
+			gtk_text_region_destroy (region, TRUE);
 		}
 
 		while (basic_backward_search (search, &iter, match_start, match_end, &limit))
@@ -2231,7 +2238,7 @@ smart_backward_search_step (GtkSourceSearchContext *search,
 		scan_region_forward (search, region);
 	}
 
-	gtk_text_region_destroy (region);
+	gtk_text_region_destroy (region, TRUE);
 
 	return FALSE;
 }
@@ -2291,12 +2298,21 @@ add_subregion_to_scan (GtkSourceSearchContext *search,
 	});
 
 	install_idle_scan (search);
+
+	/* The highlighting can be modified a bit backward and forward the
+	 * region.
+	 */
+	gtk_text_iter_backward_lines (&start, search->priv->text_nb_lines);
+	gtk_text_iter_forward_lines (&end, search->priv->text_nb_lines);
+
+	g_signal_emit_by_name (search->priv->buffer, "highlight-updated", &start, &end);
 }
 
 static void
 update_regex (GtkSourceSearchContext *search)
 {
 	gboolean regex_error_changed = FALSE;
+	GtkSourceRegexSearchState new_regex_state = GTK_SOURCE_REGEX_SEARCH_NO_ERROR;
 	const gchar *search_text = gtk_source_search_settings_get_search_text (search->priv->settings);
 
 	if (search->priv->regex != NULL)
@@ -2337,6 +2353,7 @@ update_regex (GtkSourceSearchContext *search)
 
 		if (search->priv->regex_error != NULL)
 		{
+			new_regex_state = GTK_SOURCE_REGEX_SEARCH_COMPILATION_ERROR;
 			regex_error_changed = TRUE;
 		}
 
@@ -2346,6 +2363,12 @@ update_regex (GtkSourceSearchContext *search)
 		}
 	}
 
+	if (search->priv->regex_state != new_regex_state)
+	{
+		search->priv->regex_state = new_regex_state;
+		g_object_notify (G_OBJECT (search), "regex-state");
+	}
+
 	if (regex_error_changed)
 	{
 		g_object_notify (G_OBJECT (search), "regex-error");
@@ -2476,15 +2499,9 @@ set_buffer (GtkSourceSearchContext *search,
 	    GtkSourceBuffer        *buffer)
 {
 	g_assert (search->priv->buffer == NULL);
-	g_assert (search->priv->tag_table == NULL);
 
 	search->priv->buffer = GTK_TEXT_BUFFER (buffer);
-
-	g_object_add_weak_pointer (G_OBJECT (buffer),
-				   (gpointer *)&search->priv->buffer);
-
-	search->priv->tag_table = gtk_text_buffer_get_tag_table (search->priv->buffer);
-	g_object_ref (search->priv->tag_table);
+	g_object_ref (search->priv->buffer);
 
 	g_signal_connect_object (buffer,
 				 "insert-text",
@@ -2596,24 +2613,16 @@ gtk_source_search_context_dispose (GObject *object)
 
 	clear_search (search);
 
-	if (search->priv->found_tag != NULL &&
-	    search->priv->tag_table != NULL)
+	if (search->priv->found_tag != NULL)
 	{
-		gtk_text_tag_table_remove (search->priv->tag_table,
-					   search->priv->found_tag);
+		GtkTextTagTable *tag_table = gtk_text_buffer_get_tag_table (search->priv->buffer);
+
+		gtk_text_tag_table_remove (tag_table, search->priv->found_tag);
 
 		g_clear_object (&search->priv->found_tag);
-		g_clear_object (&search->priv->tag_table);
-	}
-
-	if (search->priv->buffer != NULL)
-	{
-		g_object_remove_weak_pointer (G_OBJECT (search->priv->buffer),
-					      (gpointer *)&search->priv->buffer);
-
-		search->priv->buffer = NULL;
 	}
 
+	g_clear_object (&search->priv->buffer);
 	g_clear_object (&search->priv->settings);
 
 	G_OBJECT_CLASS (gtk_source_search_context_parent_class)->dispose (object);
@@ -2671,6 +2680,10 @@ gtk_source_search_context_get_property (GObject    *object,
 			g_value_set_pointer (value, gtk_source_search_context_get_regex_error (search));
 			break;
 
+		case PROP_REGEX_STATE:
+			g_value_set_enum (value, gtk_source_search_context_get_regex_state (search));
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -2799,12 +2812,30 @@ gtk_source_search_context_class_init (GtkSourceSearchContextClass *klass)
 							       _("Regex error"),
 							       _("Regular expression error"),
 							       G_PARAM_READABLE));
+
+	/**
+	 * GtkSourceSearchContext:regex-state:
+	 *
+	 * The regex search state.
+	 *
+	 * Since: 3.10
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_REGEX_STATE,
+					 g_param_spec_enum ("regex-state",
+							    _("Regex state"),
+							    _("State of the regular expression search"),
+							    GTK_SOURCE_TYPE_REGEX_SEARCH_STATE,
+							    GTK_SOURCE_REGEX_SEARCH_NO_ERROR,
+							    G_PARAM_READABLE));
 }
 
 static void
 gtk_source_search_context_init (GtkSourceSearchContext *search)
 {
 	search->priv = gtk_source_search_context_get_instance_private (search);
+
+	search->priv->regex_state = GTK_SOURCE_REGEX_SEARCH_NO_ERROR;
 }
 
 /**
@@ -2979,6 +3010,21 @@ gtk_source_search_context_get_regex_error (GtkSourceSearchContext *search)
 	return g_error_copy (search->priv->regex_error);
 }
 
+/**
+ * gtk_source_search_context_get_regex_state:
+ * @search: a #GtkSourceSearchContext.
+ *
+ * Returns: the regex search state.
+ * Since: 3.10
+ */
+GtkSourceRegexSearchState
+gtk_source_search_context_get_regex_state (GtkSourceSearchContext *search)
+{
+	g_return_val_if_fail (GTK_SOURCE_IS_SEARCH_CONTEXT (search), GTK_SOURCE_REGEX_SEARCH_NO_ERROR);
+
+	return search->priv->regex_state;
+}
+
 /**
  * gtk_source_search_context_get_occurrences_count:
  * @search: a #GtkSourceSearchContext.
@@ -3005,12 +3051,11 @@ gtk_source_search_context_get_occurrences_count (GtkSourceSearchContext *search)
  * @match_end: the end of the occurrence.
  *
  * Gets the position of a search occurrence. If the buffer is not already fully
- * scanned, the position may be unknown, and -1 is returned. If 0 is returned,
- * it means that this part of the buffer has already been scanned, and that
- * @match_start and @match_end don't delimit an occurrence.
+ * scanned, the position may be unknown, and -1 is returned. Therefore you
+ * should call this function when you know that the buffer is fully scanned.
  *
  * Returns: the position of the search occurrence. The first occurrence has the
- * position 1 (not 0). Returns 0 if @match_start and @match_end don't delimit
+ * position 1 (not 0). Returns 0 if @match_start and @match_end doesn't delimit
  * an occurrence. Returns -1 if the position is not yet known.
  *
  * Since: 3.10
@@ -3025,8 +3070,6 @@ gtk_source_search_context_get_occurrence_position (GtkSourceSearchContext *searc
 	GtkTextIter iter;
 	gboolean found;
 	gint position = 0;
-	GtkTextRegion *region;
-	gboolean empty;
 
 	g_return_val_if_fail (GTK_SOURCE_IS_SEARCH_CONTEXT (search), -1);
 	g_return_val_if_fail (match_start != NULL, -1);
@@ -3037,27 +3080,6 @@ gtk_source_search_context_get_occurrence_position (GtkSourceSearchContext *searc
 		return -1;
 	}
 
-	/* Verify that the [match_start; match_end] region has been scanned. */
-
-	if (search->priv->scan_region != NULL)
-	{
-		region = gtk_text_region_intersect (search->priv->scan_region,
-						    match_start,
-						    match_end);
-
-		empty = is_text_region_empty (region);
-
-		if (region != NULL)
-		{
-			gtk_text_region_destroy (region);
-		}
-
-		if (!empty)
-		{
-			return -1;
-		}
-	}
-
 	/* Verify that the occurrence is correct. */
 
 	found = smart_forward_search_without_scanning (search,
@@ -3081,15 +3103,15 @@ gtk_source_search_context_get_occurrence_position (GtkSourceSearchContext *searc
 
 	if (search->priv->scan_region != NULL)
 	{
-		region = gtk_text_region_intersect (search->priv->scan_region,
-						    &iter,
-						    match_end);
+		GtkTextRegion *region = gtk_text_region_intersect (search->priv->scan_region,
+								   &iter,
+								   match_end);
 
-		empty = is_text_region_empty (region);
+		gboolean empty = is_text_region_empty (region);
 
 		if (region != NULL)
 		{
-			gtk_text_region_destroy (region);
+			gtk_text_region_destroy (region, TRUE);
 		}
 
 		if (!empty)
@@ -3385,18 +3407,16 @@ gtk_source_search_context_backward_finish (GtkSourceSearchContext  *search,
 
 /* Returns %TRUE if replaced. */
 static gboolean
-regex_replace (GtkSourceSearchContext  *search,
-	       GtkTextIter             *match_start,
-	       GtkTextIter             *match_end,
-	       const gchar             *replace,
-	       GError                 **error)
+regex_replace (GtkSourceSearchContext *search,
+	       GtkTextIter            *match_start,
+	       GtkTextIter            *match_end,
+	       const gchar            *replace)
 {
 	GtkTextIter real_start;
 	gint start_pos;
 	gchar *subject;
 	gchar *subject_replaced;
 	GRegexMatchFlags match_options;
-	GError *tmp_error = NULL;
 
 	if (search->priv->regex == NULL ||
 	    search->priv->regex_error != NULL)
@@ -3416,13 +3436,17 @@ regex_replace (GtkSourceSearchContext  *search,
 					    start_pos,
 					    replace,
 					    match_options,
-					    &tmp_error);
+					    &search->priv->regex_error);
 
 	g_free (subject);
 
-	if (tmp_error != NULL)
+	if (search->priv->regex_error != NULL)
 	{
-		g_propagate_error (error, tmp_error);
+		search->priv->regex_state = GTK_SOURCE_REGEX_SEARCH_REPLACE_ERROR;
+
+		g_object_notify (G_OBJECT (search), "regex-error");
+		g_object_notify (G_OBJECT (search), "regex-state");
+
 		g_free (subject_replaced);
 		return FALSE;
 	}
@@ -3436,6 +3460,21 @@ regex_replace (GtkSourceSearchContext  *search,
 	return TRUE;
 }
 
+static void
+clear_replace_error (GtkSourceSearchContext *search)
+{
+	if (search->priv->regex_state == GTK_SOURCE_REGEX_SEARCH_REPLACE_ERROR)
+	{
+		search->priv->regex_state = GTK_SOURCE_REGEX_SEARCH_NO_ERROR;
+
+		g_error_free (search->priv->regex_error);
+		search->priv->regex_error = NULL;
+
+		g_object_notify (G_OBJECT (search), "regex-error");
+		g_object_notify (G_OBJECT (search), "regex-state");
+	}
+}
+
 /**
  * gtk_source_search_context_replace:
  * @search: a #GtkSourceSearchContext.
@@ -3443,7 +3482,6 @@ regex_replace (GtkSourceSearchContext  *search,
  * @match_end: the end of the match to replace.
  * @replace: the replacement text.
  * @replace_length: the length of @replace in bytes, or -1.
- * @error: location to a #GError, or %NULL to ignore errors.
  *
  * Replaces a search match by another text. If @match_start and @match_end
  * doesn't correspond to a search match, %FALSE is returned.
@@ -3456,12 +3494,11 @@ regex_replace (GtkSourceSearchContext  *search,
  * Since: 3.10
  */
 gboolean
-gtk_source_search_context_replace (GtkSourceSearchContext  *search,
-				   const GtkTextIter       *match_start,
-				   const GtkTextIter       *match_end,
-				   const gchar             *replace,
-				   gint                     replace_length,
-				   GError                 **error)
+gtk_source_search_context_replace (GtkSourceSearchContext *search,
+				   const GtkTextIter      *match_start,
+				   const GtkTextIter      *match_end,
+				   const gchar            *replace,
+				   gint                    replace_length)
 {
 	GtkTextIter start;
 	GtkTextIter end;
@@ -3470,13 +3507,14 @@ gtk_source_search_context_replace (GtkSourceSearchContext  *search,
 	g_return_val_if_fail (match_start != NULL, FALSE);
 	g_return_val_if_fail (match_end != NULL, FALSE);
 	g_return_val_if_fail (replace != NULL, FALSE);
-	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	if (dispose_has_run (search))
 	{
 		return FALSE;
 	}
 
+	clear_replace_error (search);
+
 	if (!smart_forward_search (search, match_start, &start, &end))
 	{
 		return FALSE;
@@ -3490,7 +3528,7 @@ gtk_source_search_context_replace (GtkSourceSearchContext  *search,
 
 	if (gtk_source_search_settings_get_regex_enabled (search->priv->settings))
 	{
-		return regex_replace (search, &start, &end, replace, error);
+		return regex_replace (search, &start, &end, replace);
 	}
 
 	gtk_text_buffer_begin_user_action (search->priv->buffer);
@@ -3506,7 +3544,6 @@ gtk_source_search_context_replace (GtkSourceSearchContext  *search,
  * @search: a #GtkSourceSearchContext.
  * @replace: the replacement text.
  * @replace_length: the length of @replace in bytes, or -1.
- * @error: location to a #GError, or %NULL to ignore errors.
  *
  * Replaces all search matches by another text. It is a synchronous function, so
  * it can block the user interface.
@@ -3519,10 +3556,9 @@ gtk_source_search_context_replace (GtkSourceSearchContext  *search,
  * Since: 3.10
  */
 guint
-gtk_source_search_context_replace_all (GtkSourceSearchContext  *search,
-				       const gchar             *replace,
-				       gint                     replace_length,
-				       GError                 **error)
+gtk_source_search_context_replace_all (GtkSourceSearchContext *search,
+				       const gchar            *replace,
+				       gint                    replace_length)
 {
 	GtkTextIter iter;
 	GtkTextIter match_start;
@@ -3533,17 +3569,16 @@ gtk_source_search_context_replace_all (GtkSourceSearchContext  *search,
 
 	g_return_val_if_fail (GTK_SOURCE_IS_SEARCH_CONTEXT (search), 0);
 	g_return_val_if_fail (replace != NULL, 0);
-	g_return_val_if_fail (error == NULL || *error == NULL, 0);
 
 	if (dispose_has_run (search))
 	{
 		return 0;
 	}
 
+	clear_replace_error (search);
+
 	if (gtk_source_search_settings_get_regex_enabled (search->priv->settings))
 	{
-		GError *tmp_error = NULL;
-
 		if (search->priv->regex == NULL ||
 		    search->priv->regex_error != NULL)
 		{
@@ -3552,11 +3587,15 @@ gtk_source_search_context_replace_all (GtkSourceSearchContext  *search,
 
 		g_regex_check_replacement (replace,
 					   &has_regex_references,
-					   &tmp_error);
+					   &search->priv->regex_error);
 
-		if (tmp_error != NULL)
+		if (search->priv->regex_error != NULL)
 		{
-			g_propagate_error (error, tmp_error);
+			search->priv->regex_state = GTK_SOURCE_REGEX_SEARCH_REPLACE_ERROR;
+
+			g_object_notify (G_OBJECT (search), "regex-error");
+			g_object_notify (G_OBJECT (search), "regex-state");
+
 			return 0;
 		}
 	}
@@ -3578,20 +3617,25 @@ gtk_source_search_context_replace_all (GtkSourceSearchContext  *search,
 
 	while (smart_forward_search (search, &iter, &match_start, &match_end))
 	{
+		gboolean replaced = FALSE;
+
 		if (has_regex_references)
 		{
-			if (!regex_replace (search, &match_start, &match_end, replace, error))
-			{
-				break;
-			}
+			replaced = regex_replace (search, &match_start, &match_end, replace);
 		}
 		else
 		{
 			gtk_text_buffer_delete (search->priv->buffer, &match_start, &match_end);
 			gtk_text_buffer_insert (search->priv->buffer, &match_end, replace, replace_length);
+
+			replaced = TRUE;
+		}
+
+		if (replaced)
+		{
+			nb_matches_replaced++;
 		}
 
-		nb_matches_replaced++;
 		iter = match_end;
 	}
 
@@ -3637,7 +3681,7 @@ _gtk_source_search_context_update_highlight (GtkSourceSearchContext *search,
 	{
 		if (region_to_highlight != NULL)
 		{
-			gtk_text_region_destroy (region_to_highlight);
+			gtk_text_region_destroy (region_to_highlight, TRUE);
 		}
 
 		return;
@@ -3652,7 +3696,7 @@ _gtk_source_search_context_update_highlight (GtkSourceSearchContext *search,
 			 * means that the visible region has changed. So we can
 			 * destroy the old high_priority_region.
 			 */
-			gtk_text_region_destroy (search->priv->high_priority_region);
+			gtk_text_region_destroy (search->priv->high_priority_region, TRUE);
 		}
 
 		search->priv->high_priority_region = region_to_highlight;
@@ -3674,6 +3718,6 @@ _gtk_source_search_context_update_highlight (GtkSourceSearchContext *search,
 	else
 	{
 		scan_all_region (search, region_to_highlight);
-		gtk_text_region_destroy (region_to_highlight);
+		gtk_text_region_destroy (region_to_highlight, TRUE);
 	}
 }
