@@ -33,6 +33,12 @@ G_BEGIN_DECLS
 GType gtk_source_gutter_renderer_alignment_mode_get_type (void);
 G_END_DECLS
 
+#include <gtksourceview/gtksourcesearchcontext.h>
+G_BEGIN_DECLS
+#define GTK_SOURCE_TYPE_REGEX_SEARCH_STATE gtk_source_regex_search_state_get_type()
+GType gtk_source_regex_search_state_get_type (void);
+G_END_DECLS
+
 #include <gtksourceview/gtksourceview.h>
 G_BEGIN_DECLS
 #define GTK_SOURCE_TYPE_VIEW_GUTTER_POSITION gtk_source_view_gutter_position_get_type()
