@@ -41,35 +41,6 @@ static void check_async_search_results (GtkSourceSearchContext *context,
 					gboolean                forward,
 					gboolean                start_check);
 
-/* If we are running from the source dir (e.g. during make check)
- * we override the path to read from the data dir.
- */
-static void
-init_style_scheme_manager (void)
-{
-	gchar *dir;
-
-	dir = g_build_filename (TOP_SRCDIR, "data", "styles", NULL);
-
-	if (g_file_test (dir, G_FILE_TEST_IS_DIR))
-	{
-		GtkSourceStyleSchemeManager *manager;
-		gchar **dirs;
-
-		manager = gtk_source_style_scheme_manager_get_default ();
-
-		dirs = g_new0 (gchar *, 2);
-		dirs[0] = dir;
-
-		gtk_source_style_scheme_manager_set_search_path (manager, dirs);
-		g_strfreev (dirs);
-	}
-	else
-	{
-		g_free (dir);
-	}
-}
-
 static void
 flush_queue (void)
 {
@@ -612,6 +583,7 @@ test_async_forward_search_normal (void)
 
 	gtk_text_buffer_set_text (text_buffer, "aaaa", -1);
 	gtk_source_search_settings_set_search_text (settings, "aa");
+	gtk_source_search_settings_set_wrap_around (settings, FALSE);
 
 	check_async_search_results (context, results, TRUE, TRUE);
 
@@ -733,6 +705,7 @@ test_async_backward_search_normal (void)
 
 	gtk_text_buffer_set_text (text_buffer, "aaaa", -1);
 	gtk_source_search_settings_set_search_text (settings, "aa");
+	gtk_source_search_settings_set_wrap_around (settings, FALSE);
 
 	check_async_search_results (context, results, FALSE, TRUE);
 
@@ -868,13 +841,13 @@ test_replace (void)
 	gtk_text_buffer_get_iter_at_offset (text_buffer, &start, 1);
 	gtk_text_buffer_get_iter_at_offset (text_buffer, &end, 3);
 
-	replaced = gtk_source_search_context_replace (context, &start, &end, "bb", 2, NULL);
+	replaced = gtk_source_search_context_replace (context, &start, &end, "bb", 2);
 	g_assert (!replaced);
 
 	gtk_text_buffer_get_iter_at_offset (text_buffer, &start, 2);
 	gtk_text_buffer_get_iter_at_offset (text_buffer, &end, 4);
 
-	replaced = gtk_source_search_context_replace (context, &start, &end, "bb", 2, NULL);
+	replaced = gtk_source_search_context_replace (context, &start, &end, "bb", 2);
 	g_assert (replaced);
 
 	gtk_text_buffer_get_start_iter (text_buffer, &start);
@@ -905,7 +878,7 @@ test_replace_all (void)
 	gtk_source_search_settings_set_search_text (settings, "aa");
 	flush_queue ();
 
-	nb_replacements = gtk_source_search_context_replace_all (context, "bb", 2, NULL);
+	nb_replacements = gtk_source_search_context_replace_all (context, "bb", 2);
 	g_assert_cmpint (nb_replacements, ==, 2);
 
 	gtk_text_buffer_get_start_iter (text_buffer, &start);
@@ -960,7 +933,7 @@ test_regex (void)
 
 	gtk_text_buffer_get_start_iter (text_buffer, &start);
 	gtk_text_buffer_get_end_iter (text_buffer, &end);
-	gtk_source_search_context_replace (context, &start, &end, "\\2#\\1", -1, NULL);
+	gtk_source_search_context_replace (context, &start, &end, "\\2#\\1", -1);
 
 	gtk_text_buffer_get_start_iter (text_buffer, &start);
 	gtk_text_buffer_get_end_iter (text_buffer, &end);
@@ -971,7 +944,7 @@ test_regex (void)
 	/* Test replace all */
 
 	gtk_text_buffer_set_text (text_buffer, "aa#bb cc#dd", -1);
-	gtk_source_search_context_replace_all (context, "\\2#\\1", -1, NULL);
+	gtk_source_search_context_replace_all (context, "\\2#\\1", -1);
 
 	gtk_text_buffer_get_start_iter (text_buffer, &start);
 	gtk_text_buffer_get_end_iter (text_buffer, &end);
@@ -1029,8 +1002,6 @@ main (int argc, char **argv)
 {
 	gtk_test_init (&argc, &argv);
 
-	init_style_scheme_manager ();
-
 	g_test_add_func ("/Search/occurrences-count/simple", test_occurrences_count_simple);
 	g_test_add_func ("/Search/occurrences-count/with-insert", test_occurrences_count_with_insert);
 	g_test_add_func ("/Search/occurrences-count/with-delete", test_occurrences_count_with_delete);
